//place a sound in 2D space 
//using the positions of sound and listener 
//jrh 2003, see also AdC tutorial for spatialisation technique

LinPanXY {
	var <>soundPos, <>listeningPos; //points
	var <>listeningDirection; //in radians
	var <>earDist; 
	
	*new { arg soundPos, listeningPos, listeningDirection, earDist=0.25;
			^super.newCopyArgs(soundPos, listeningPos, listeningDirection, earDist)
	}
	
	*ar { arg in, soundPos, listeningPos, listeningDirection, earDist=0.25, ampCurve=2;
		^this.new(soundPos, listeningPos, listeningDirection, earDist).ar(in, ampCurve)
	}
	ar { arg in, ampCurve=2;
		var panPos;
		panPos = 0.5 * this.panPos + 0.5;
		^in * min(1, (this.distances ** ampCurve).reciprocal) * [panPos, 1-panPos] * 2;
	}
	
	relativeDirection {
		^(soundPos-listeningPos).theta - pi + listeningDirection
	}
	
	panPos {
		^this.relativeDirection.sin
	}
	
	distances {
		var dx, dy, ddx, ddy;
		dx = cos(listeningDirection)*earDist;
		dy = sin(listeningDirection)*earDist;
		ddx = soundPos.x - listeningPos.x;
		ddy = soundPos.y - listeningPos.y;
		^[
			hypot(ddx + dx, ddy + dy),
			hypot(ddx - dx, ddy - dy)
		]
	}
}

DelayPanXY : LinPanXY {
	
	*ar { arg in, soundPos, listeningPos, listeningDirection, earDist=0.25, 
			maxDist=100, speedOfSound=344, ampCurve=2;
		^this.new(soundPos, listeningPos, listeningDirection, earDist=0.25)
				.ar(in, maxDist=100, speedOfSound=344, ampCurve);
	}
	
	ar { arg in, maxDist=100, speedOfSound=344, ampCurve=2;
		var delay, maxDelay, amps, panPos;
		delay = min(this.distances, maxDist) / speedOfSound;
		maxDelay = maxDist / speedOfSound;
		panPos = 0.5 * this.panPos + 0.5;
		amps = min(1, (this.distances ** ampCurve).reciprocal) * [panPos, 1-panPos] * 2;
		^DelayL.ar(in, maxDelay, delay, amps);
	}

}


